=== modified file 'bzrlib/cache_utf8.py'
--- bzrlib/cache_utf8.py	2007-02-08 23:10:37 +0000
+++ bzrlib/cache_utf8.py	2007-02-09 16:41:00 +0000
@@ -88,6 +88,19 @@
     return encode(decode(utf8_str))
 
 
+def get_cached_ascii(ascii_str,
+                     _uni_to_utf8=_unicode_to_utf8_map,
+                     _utf8_to_uni=_utf8_to_unicode_map):
+    """This is a string which is identical in utf-8 and unicode."""
+    # We don't need to do any encoding, but we want _utf8_to_uni to return a
+    # real Unicode string. Unicode and plain strings of this type will have the
+    # same hash, so we can just use it as the key in _uni_to_utf8, but we need
+    # the return value to be different in _utf8_to_uni
+    ascii_str = _uni_to_utf8.setdefault(ascii_str, ascii_str)
+    _utf8_to_uni.setdefault(ascii_str, unicode(ascii_str))
+    return ascii_str
+
+
 def clear_encoding_cache():
     """Clear the encoding and decoding caches"""
     _unicode_to_utf8_map.clear()

=== modified file 'bzrlib/tests/test_cache_utf8.py'
--- bzrlib/tests/test_cache_utf8.py	2007-02-08 23:10:37 +0000
+++ bzrlib/tests/test_cache_utf8.py	2007-02-09 16:41:00 +0000
@@ -78,8 +78,8 @@
         xp = cache_utf8.get_cached_unicode(x)
         yp = cache_utf8.get_cached_unicode(y)
 
-        self.failUnless(xp is x)
-        self.failUnless(xp is yp)
+        self.assertIs(xp, x)
+        self.assertIs(xp, yp)
 
     def test_cached_utf8(self):
         x = u'\xb5yy\xe5zz'.encode('utf8')
@@ -88,5 +88,24 @@
         xp = cache_utf8.get_cached_utf8(x)
         yp = cache_utf8.get_cached_utf8(y)
 
-        self.failUnless(xp is x)
-        self.failUnless(xp is yp)
+        self.assertIs(xp, x)
+        self.assertIs(xp, yp)
+
+    def test_cached_ascii(self):
+        x = '%s %s' % ('simple', 'text')
+        y = '%s %s' % ('simple', 'text')
+        self.failIf(x is y)
+        xp = cache_utf8.get_cached_ascii(x)
+        yp = cache_utf8.get_cached_ascii(y)
+
+        self.assertIs(xp, x)
+        self.assertIs(xp, yp)
+
+        # after caching, encode and decode should also return the right
+        # objects.
+        uni_x = cache_utf8.decode(x)
+        self.assertEqual(u'simple text', uni_x)
+        self.assertIsInstance(uni_x, unicode)
+
+        utf8_x = cache_utf8.encode(uni_x)
+        self.assertIs(utf8_x, x)

